#!/usr/bin/env python

"""
Java Servlet classes.
"""

import Generic
from StringIO import StringIO
from Helpers.Request import Cookie
import javax.servlet.http.Cookie

class Stream:

    """
    Wrapper around java.io.BufferedReader.
    """

    def __init__(self, stream):

        "Initialise the stream with the given underlying 'stream'."

        self.stream = stream

    def read(self):

        "Read the entire message, returning it as a string."

        characters = StringIO()
        while 1:
            c = self.stream.read()
            if c == -1:
                return characters.getvalue()
            else:
                characters.write(chr(c))

    def readline(self):

        "Read a line from the stream, returning it as a string."

        return self.stream.readLine()

class Transaction(Generic.Transaction):

    """
    Java Servlet transaction interface.
    """

    def __init__(self, request, response):

        """
        Initialise the transaction using the Java Servlet HTTP 'request' and
        'response'.
        """

        self.request = request
        self.response = response
        self.status = None
        self.user = None

        # Remember the cookies received in the request.
        # NOTE: Discarding much of the information received.

        self.cookies_in = {}
        for cookie in self.request.getCookies():
            cookie_name = cookie.getName()
            self.cookies_in[cookie_name] = Cookie(cookie_name, cookie.getValue())

    def commit(self):

        """
        A special method, synchronising the transaction with framework-specific
        objects.
        """

        self.get_response_stream().close()

    # Request-related methods.

    def get_request_stream(self):

        """
        A framework-specific method which returns the request stream for
        the transaction.
        """

        return Stream(self.request.getReader())

    def get_request_method(self):

        """
        A framework-specific method which gets the request method.
        """

        return self.request.getMethod()

    def get_headers(self):

        """
        A framework-specific method which returns all request headers as a
        dictionary-like object mapping header names to values.
        NOTE: If duplicate header names are permitted, then this interface will
        NOTE: need to change.
        """

        headers = {}
        header_names = self.request.getHeaderNames()
        if header_names:
            for header_name in header_names:

                # NOTE: Retrieve only a single value (not using getHeaders).

                headers[header_name] = self.request.getHeader(header_name)

        return headers

    def get_header_values(self, key):

        """
        A framework-specific method which returns a list of all request header
        values associated with the given 'key'. Note that according to RFC 2616,
        'key' is treated as a case-insensitive string.
        """

        return self.request.getHeaders(key)

    def get_content_type(self):

        """
        A framework-specific method which gets the content type specified on the
        request, along with the charset employed.
        """

        content_types = self.get_header_values("Content-Type")
        if len(content_types) >= 1:
            return self.parse_content_type(content_types[0])
        else:
            return None

    def get_content_charsets(self):

        """
        Returns the character set preferences.
        """

        accept_charsets = self.get_header_values("Accept-Charset")
        if len(accept_charsets) >= 1:
            return self.parse_content_preferences(accept_charsets[0])
        else:
            return None

    def get_content_languages(self):

        """
        A framework-specific method which extracts language information from
        the transaction.
        """

        accept_languages = self.get_header_values("Accept-Language")
        if len(accept_languages) >= 1:
            return self.parse_content_preferences(accept_languages[0])
        else:
            return None

    def get_path(self):

        """
        A framework-specific method which gets the entire path from the request.
        """

        return self.request.getServletPath()

    def get_path_info(self):

        """
        A framework-specific method which gets the "path info" (the part of the
        URL after the resource name handling the current request) from the
        request.
        """

        return self.request.getPathInfo()

    def get_query_string(self):

        """
        A framework-specific method which gets the query string from the path in
        the request.
        """

        return self.request.getQueryString()

    # Higher level request-related methods.

    def get_fields_from_path(self):

        """
        A framework-specific method which extracts the form fields from the
        path specified in the transaction. The underlying framework may refuse
        to supply fields from the path if handling a POST transaction.

        Returns a dictionary mapping field names to lists of values (even if a
        single value is associated with any given field name).

        NOTE: There may not be a reliable means of extracting only the fields
        NOTE: from the path.
        """

        return self.get_fields_from_body()

    def get_fields_from_body(self):

        """
        A framework-specific method which extracts the form fields from the
        message body in the transaction.

        Returns a dictionary mapping field names to lists of values (even if a
        single value is associated with any given field name).

        NOTE: There may not be a reliable means of extracting only the fields
        NOTE: from the message body.
        """

        parameter_map = self.request.getParameterMap()
        fields = {}
        for key in parameter_map.keySet():
            fields[key] = parameter_map[key]
        return fields

    def get_user(self):

        """
        A framework-specific method which extracts user information from the
        transaction.

        Returns a username as a string or None if no user is defined.
        """

        if self.user is not None:
            return self.user
        else:
            return self.request.getRemoteUser()

    def get_cookies(self):

        """
        A framework-specific method which obtains cookie information from the
        request.

        Returns a dictionary mapping cookie names to cookie objects.
        """

        return self.cookies_in

    def get_cookie(self, cookie_name):

        """
        A framework-specific method which obtains cookie information from the
        request.

        Returns a cookie object for the given 'cookie_name' or None if no such
        cookie exists.
        """

        return self.cookies_in.get(cookie_name)

    # Response-related methods.

    def get_response_stream(self):

        """
        A framework-specific method which returns the response stream for
        the transaction.
        """

        return self.response.getWriter()

    def get_response_code(self):

        """
        Get the response code associated with the transaction. If no response
        code is defined, None is returned.
        """

        return self.status

    def set_response_code(self, response_code):

        """
        Set the 'response_code' using a numeric constant defined in the HTTP
        specification.
        """

        self.status = response_code
        self.response.setStatus(self.status)

    def set_header_value(self, header, value):

        """
        Set the HTTP 'header' with the given 'value'.
        """

        self.response.setHeader(self.format_header_value(header), self.format_header_value(value))

    def set_content_type(self, content_type):

        """
        A framework-specific method which sets the 'content_type' for the
        response.
        """

        return self.response.setHeader("Content-Type", self.format_content_type(content_type))

    # Higher level response-related methods.

    def set_cookie(self, cookie):

        """
        A framework-specific method which stores the given 'cookie' object in
        the response.
        """

        new_cookie = javax.servlet.http.Cookie(cookie.name, cookie.value)
        self.response.addCookie(new_cookie)

    def set_cookie_value(self, name, value, path=None, expires=None):

        """
        A framework-specific method which stores a cookie with the given 'name'
        and 'value' in the response.

        The optional 'path' is a string which specifies the scope of the cookie,
        and the optional 'expires' parameter is a value compatible with the
        time.time function, and indicates the expiry date/time of the cookie.
        """

        cookie = javax.servlet.http.Cookie(name, value)
        if path is not None:
            cookie.setPath(path)

        # NOTE: The expires parameter seems not to be supported.

        self.response.addCookie(cookie)

    def delete_cookie(self, cookie_name):

        """
        A framework-specific method which adds to the response a request that
        the cookie with the given 'cookie_name' be deleted/discarded by the
        client.
        """

        # Create a special cookie, given that we do not know whether the browser
        # has been sent the cookie or not.
        # NOTE: Magic discovered in Webware.

        cookie = javax.servlet.http.Cookie(cookie_name, "")
        cookie.setPath("/")
        cookie.setMaxAge(0)
        self.response.addCookie(cookie)

    # Application-specific methods.

    def set_user(self, username):

        """
        An application-specific method which sets the user information with
        'username' in the transaction. This affects subsequent calls to
        'get_user'.
        """

        self.user = username

# vim: tabstop=4 expandtab shiftwidth=4
